#!/usr/bin/env python
# coding: utf-8

# # Object Detection Inference using TF Lite Runtime
# 
# 
# ## Inference using pre-trained models from open source Tensorflow library
# In this example notebook, we describe how to use a pre-trained Object Detection model for inference using the TF Lite Runtime interface.
#    - We perform inference on a few sample images.
#    - We also describe the input preprocessing and output postprocessing steps, demonstrate how to collect various benchmarking statistics and how to visualize the data.
# 
# 
# ## Object Detection  
# 
# Object Detection is a popular computer vision algorithm used in many applications such as Person Detection and Vehicle detection. The image below shows object detection results on few sample images.
# 
# 

# In[1]:


import os
import cv2
import numpy as np
import tflite_runtime.interpreter as tflite


import time
import ipywidgets as widgets

from scripts.myutils import make_label_dict


# In[ ]:





# ## Define utility function to preprocess input images
# 
# Below, we define a utility function to preprocess images for the model. This function takes a path as input, loads the image and preprocesses the images as required by the model. The steps below are shown as a reference (no user action required):
# 
#  1. Load image
#  2. Convert BGR image to RGB
#  3. Scale image
#  4. Apply per-channel pixel scaling and mean subtraction
#  5. Convert RGB Image to BGR. 
#  6. Convert the image to NCHW format
# 
# 
# - The input arguments of this utility function is selected automatically by this notebook based on the model selected in the drop-down

# In[2]:


def preprocess(image_path, size, mean, scale, layout, reverse_channels):
    # Step 1
    img = cv2.imread(image_path)
    
    # Step 2
    img = img[:,:,::-1]
    
    # Step 3
    img = cv2.resize(img, (size[1], size[0]), interpolation=cv2.INTER_CUBIC)
     
    # Step 4
    img = img.astype('float32')
    for mean, scale, ch in zip(mean, scale, range(img.shape[2])):
            img[:,:,ch] = ((img.astype('float32')[:,:,ch] - mean) * scale)
    # Step 5
    if reverse_channels:
        img = img[:,:,::-1]
        
    # Step 6
    if layout == 'NCHW':
        img = np.expand_dims(np.transpose(img, (2,0,1)),axis=0)
    else:
        img = np.expand_dims(img,axis=0)
    
    return img


# ## Create the model using the stored artifacts

# In[3]:



# Load TFLite model and allocate tensors.
#interpreter = tf.Interpreter(model_path="ssd_mobilenet_v1_1_metadata_1/ssd_mobilenet_v1_1_metadata_1.tflite")

label_dict = make_label_dict('TFL-OD-200-ssd-mobV1-coco-mlperf-300x300/labels.txt')

# Use a pre-trained model compiled using TIDL
tflite_model_path = 'TFL-OD-200-ssd-mobV1-coco-mlperf-300x300/ssd_mobilenet_v1_coco_2018_01_28.tflite'

interpreter = tflite.Interpreter(model_path=tflite_model_path)

interpreter.allocate_tensors()

input_details = interpreter.get_input_details()
output_details = interpreter.get_output_details()


# ## Perform inference
# 
# ### Preprocessing and Inference
#   - We perform inference on a set of images from the `/ip` directory. 
#   - We use a loop to preprocess the selected images, and provide them as the input to the network.
# 
# ### Postprocessing and Visualization
#  - Once the inference results are available, we postpocess the results and visualize the inferred classes for each of the input images.
#  - Object Detection models return results as a list (i.e. `numpy.ndarray`) with a minimum length of 3. Each element in this list contains, the detected object class ID, the probability of the detection and the bounding box co-ordinates.
#  - We use the `det_box_overlay()` function to draw detected boxes on the each object with unique colors for each class ID.
#  - Then, in this notebook, we use *matplotlib* to plot the original images and the corresponding results.

# In[4]:


from scripts.myutils import get_preproc_props

# use results from the past inferences
#images = [
#    ('sample-images/ADE_val_00001801.jpg', 221),
#    ('sample-images/ti_lindau_I00000.jpg', 222)
#]
images = [
'ip/dog_cat.jpg',
]
size = [300, 300]
mean = [128.0, 128.0, 128.0]
scale = [0.0078125, 0.0078125, 0.0078125]
layout = 'NHWC'
reverse_channels = False
print(f'Image size: {size}')


# In[5]:


import tqdm
import matplotlib.pyplot as plt
from PIL import Image
from scripts.myutils import det_box_overlay

plt.figure(figsize=(20,10))


for num in tqdm.trange(len(images)):
    image_file = images[num]
    img  = Image.open(image_file).convert('RGB')
    ax = plt.subplot()
    
    img_in = preprocess(image_file , size, mean, scale, layout, reverse_channels)
    if not input_details[0]['dtype'] == np.float32:
        img_in = np.uint8(img_in)

    start = time.time()        
    interpreter.set_tensor(input_details[0]['index'], img_in)    
    interpreter.invoke()
    res = [interpreter.get_tensor(output_detail['index']) for output_detail in output_details]
    end = time.time()
    print ("Elaspsed time in seconds: ", end-start)
    
    org_size = img.size
    img = det_box_overlay(res, img.resize((size[1], size[0])), 0.5, label_dict)
    img = img.resize(org_size)
    ax.imshow(img)
    
plt.show()


# In[ ]:




