#!/usr/bin/env python
# coding: utf-8

# # Step2: Running the same step1 program on Jacinto Edge AI Hardware (Using Cloud Tools)
# 
# 
# # Object Detection Inference using TF Lite Runtime
# 
# 

# In[1]:


import os
import cv2
import numpy as np
import tflite_runtime.interpreter as tflite

import time
import ipywidgets as widgets
from scripts.myutils import get_eval_configs
from scripts.myutils import make_label_dict

from PIL import Image, ImageFont, ImageDraw, ImageEnhance
from io import BytesIO


# ## Define utility function to preprocess input images
# 
# Below, we define a utility function to preprocess images for the model. This function takes a path as input, loads the image and preprocesses the images as required by the model. The steps below are shown as a reference (no user action required):
# 
#  1. Load image
#  2. Convert BGR image to RGB
#  3. Scale image
#  4. Apply per-channel pixel scaling and mean subtraction
#  5. Convert RGB Image to BGR. 
#  6. Convert the image to NCHW format
# 
# 
# - The input arguments of this utility function is selected automatically by this notebook based on the model selected in the drop-down

# In[2]:


def preprocess(image_path, size, mean, scale, layout, reverse_channels):
    # Step 1
    img = cv2.imread(image_path)
    
    # Step 2
    img = img[:,:,::-1]
    
    # Step 3
    img = cv2.resize(img, (size[1], size[0]), interpolation=cv2.INTER_CUBIC)
     
    # Step 4
    img = img.astype('float32')
    for mean, scale, ch in zip(mean, scale, range(img.shape[2])):
            img[:,:,ch] = ((img.astype('float32')[:,:,ch] - mean) * scale)
    # Step 5
    if reverse_channels:
        img = img[:,:,::-1]
        
    # Step 6
    if layout == 'NCHW':
        img = np.expand_dims(np.transpose(img, (2,0,1)),axis=0)
    else:
        img = np.expand_dims(img,axis=0)
    
    return img


# ## Load the Tflite model and allocate tensors

# In[3]:


import tflite_runtime.interpreter as tflite

# Load TFLite model and allocate tensors.

# Make a dictionary of labels and class values
label_dict = make_label_dict('TFL-OD-200-ssd-mobV1-coco-mlperf-300x300/labels.txt')

# Use a pre-trained model compiled using TIDL
tflite_model_path ='TFL-OD-200-ssd-mobV1-coco-mlperf-300x300/ssd_mobilenet_v1_coco_2018_01_28.tflite'
# Bypass the delegate. Run it directly on the ARM
interpreter = tflite.Interpreter(model_path=tflite_model_path)

interpreter.allocate_tensors()

input_details = interpreter.get_input_details()
output_details = interpreter.get_output_details()


# ## Perform inference
# 
# ### Preprocessing and Inference
#   - We perform inference on a set of images from the `/sample-images` directory. 
#   - We use a loop to preprocess the selected images, and provide them as the input to the network.
# 
# ### Postprocessing and Visualization
#  - Once the inference results are available, we postpocess the results and visualize the inferred classes for each of the input images.
#  - Object Detection models return results as a list (i.e. `numpy.ndarray`) with a minimum length of 3. Each element in this list contains, the detected object class ID, the probability of the detection and the bounding box co-ordinates.
#  - We use the `det_box_overlay()` function to draw detected boxes on the each object with unique colors for each class ID.
#  - Then, in this notebook, we use *matplotlib* to plot the original images and the corresponding results.

# In[4]:


from scripts.myutils import get_preproc_props


images = [
'ip/dog_cat.jpg',
]
size = [300, 300]
mean = [128.0, 128.0, 128.0]
scale = [0.0078125, 0.0078125, 0.0078125]
layout = 'NHWC'
reverse_channels = False


# In[5]:


import tqdm
import matplotlib.pyplot as plt
from PIL import Image
from scripts.myutils import det_box_overlay

    


plt.figure(figsize=(20,10))

# use results from the past inferences

for num in tqdm.trange(len(images)):
    image_file = images[num]
    img  = Image.open(image_file).convert('RGB')
    ax = plt.subplot()
    
    img_in = preprocess(image_file , size, mean, scale, layout, reverse_channels)
    if not input_details[0]['dtype'] == np.float32:
        img_in = np.uint8(img_in)

    start = time.time()        
    interpreter.set_tensor(input_details[0]['index'], img_in)    
    interpreter.invoke()
    res = [interpreter.get_tensor(output_detail['index']) for output_detail in output_details]
    end = time.time()
    print ("Elaspsed time in seconds: ", end-start)
    
    
    org_size = img.size
    img = det_box_overlay(res, img.resize((size[1], size[0])), 0.5, label_dict)
    img = img.resize(org_size)
    ax.imshow(img)
    
plt.show()


# ## Plot Inference benchmarking statistics
#  - During model execution several benchmarking statistics such as timestamps at different checkpoints, DDR bandwidth are collected and stored. 
#  - The `get_TI_benchmark_data()` function can be used to collect these statistics. The statistics are collected as a dictionary of `annotations` and corresponding markers.
#  - We provide the utility function plot_TI_benchmark_data to visualize these benchmark KPIs.
# 
# <div class="alert alert-block alert-info">
# <b>Note:</b> The values represented by <i>Inferences Per Second</i> and <i>Inference Time Per Image</i> uses the total time taken by the inference except the time taken for copying inputs and outputs. In a performance oriented system, these operations can be bypassed by writing the data directly into shared memory and performing on-the-fly input / output normalization.
# </div>
# 

# In[6]:


from scripts.myutils import plot_TI_performance_data, plot_TI_DDRBW_data, get_benchmark_output

stats = interpreter.get_TI_benchmark_data()
fig, ax = plt.subplots(nrows=1, ncols=1, figsize=(10,5))
plot_TI_performance_data(stats, axis=ax)
plt.show()

tt, st, rb, wb = get_benchmark_output(stats)

print(f'SoC: J721E/DRA829/TDA4VM')
print(f' OPP:')
print(f'  Cortex-A72 @2GHZ')
print(f'  DSP C7x-MMA @1GHZ')
print(f'  DDR @4266 MT/s\n')
print(f' Inferences Per Second    : {1000.0/tt :7.2f} fps')
print(f' Inference Time Per Image : {tt :7.2f} ms')
print(f' DDR usage Per Image      : {rb+ wb : 7.2f} MB')


# In[ ]:




